/* C-SLang 2.0. cslpubl.h
   
   This header provides public data types and function prototypes
   based on user-supplied types in csl_opt.h
   
   This is the only C-SLang header to #include explicitly
   to compose a virtual machine for running a script.

   Copyright (C) 2003 MacroExpressions.com 

   Version control:
	$Date: 2004/08/07 $
	$Change: 70 $
	$Revision: #5 $
*/

#ifndef CSLPUBL_H_
#define CSLPUBL_H_

#include "cslpriv.h" /* Implementation-specific (configuration) definitions */


/*************** Data types *****************/
typedef unsigned char ubyte_t;
/* VPIO input and output function types */
typedef short InpFunc_t(unsigned ionum, reg_t *arg); /* VPIO input and output function types */
typedef short int OutpFunc_t(unsigned ionum, outfunc_arg_t value);
/* byte output "callback" function type for script export */
typedef void ByteDump_t(int c, void *arg);


/* ------ breakpoints -------- */
struct CSLbp_t{
	struct CSLbp_t *next;
	ubyte_t func_id;
	unsigned short instr;
};


/* SVIRM input buffer type */
typedef struct {
	const inpvar_t *buf;
	ubyte_t size;
} inpbuf_t;

/* SVIRM output buffer type */
typedef struct {
	outpvar_t *buf;
	ubyte_t size;
} outpbuf_t;

/* SVIRM temporaries buffer type */
typedef struct {
	tempvar_t *buf;
	ubyte_t size;
} tempbuf_t;



/* -- C-SLang Virtual Machine data type -- */
struct SVIRM_t{
	inpbuf_t inp;
	outpbuf_t outp;
	tempbuf_t tempbuf;
	InpFunc_t *infunc;
	OutpFunc_t *outfunc;
};



/*********** public interfaces ***********/
extern int CSLangBytecodeExport(CSLscript_t script, ByteDump_t * emitFunc, void *arg);
extern int CSLangInitCom(CSLangRun_t *run, CSLscript_t script, SVIRM_t const *svirm);
extern int CSLangInitExp(CSLangRun_t *run, const unsigned char *script, SVIRM_t const *svirm);
extern void CSLangSetControlStack(CSLangRun_t *run, CtlStack_t *ptr, unsigned short size);
extern int CSLangExec(CSLangRun_t *run);
/*--------- Debug interface ---------*/
extern void SCLangSetSingleStep(CSLangRun_t *run, unsigned char sstep);
extern void SCLangSetBreakpoints(CSLangRun_t *run, const CSLbp_t *breakpoints);


/* C-SLang completion codes: definitions */
/* ----- Negative codes - errors -------- */
#define CSL_CALLS_TOO_NESTED (-1)
#define CSL_UNIMPLEMENTED_OPCODE (-2)
#define CSL_NONWRITEABLE_TARGET (-4)
#define CSL_WILD_EOLOOP (-5)
#define CSL_ILLEGAL_REPEAT (-6)
#define CSL_BADINSTR (-7) /*bad instruction in script*/
#define CSL_UNIMPLEMENTED_FUNCTION (-8) /*VPIO function called which is not supplied*/
#define CSL_INDEX_LIMIT (-9) /*array limit exceeded*/
#define CSL_ZERODIVIDE (-10) /*an attempt to divide by 0*/
#define CSL_FUNCTION_INDEX (-11) /*destination function has index greater than there are in the script*/
#define CSL_CSRIPT_TOO_LONG (-12) /*script length doesn't fit 2 bytes*/

/* Demo version only */
#define CSL_DEMO_FUNC_LIMIT (-20)
#define CSL_DEMO_CODE_LIMIT (-21)
/* End Demo version only */

/* ----- Positive codes - no errors -------- */
#define CSL_DONE 1 /* Run completed; no errors encountered */
#define CSL_STEP_COMPLETED 2 /* An instruction completed in single-step mode */
#define CSL_BREAKPOINT 3     /* A breakpoint is reached */
 

/*Misc. -- in case it is not defined elsewhere*/
#ifndef NULL
 #define NULL ((void *)0)
#endif

#endif /*CSLPUBL_H_*/

