/* C-SLang 2.0. sample3.c
    SAE message processing as described in the
    manual.
    
   (C) 2003 MacroExpressions.com 

   Version control:
	$Date: 2003/12/30 $
	$Change: 35 $
	$Revision: #4 $
*/
/* Comment out the following definition if your compiler handles
   tentative forward declarations correctly
*/
#define CSL_CONST 

#include "csldef.h"
#include "cslpubl.h"
#include "sample.h"

/********************** Sample Script *******************/
ScriptStart (SAE_example)
RegStart(
   RegFunc(SAEMessage)
   RegFunc(PositiveResp)
   RegFunc(NegativeResp)
   /* mode-specific functions are registered
      in the order convenient for ComputedCall
   */
   RegFunc(Mode12)
   RegFunc(Mode14)
   RegFunc(Mode17)
   RegFunc(Mode19)
   RegFunc(Mode20)
)
Function( SAEMessage )
	Move(Literal, 0, OutpVar, 0) /* resp. length 0 means 'ignore' */
	LoadA(InpVar, 0) /* length */
	Subtract(Literal, 4)
	RetNeg	/* too short a message */
	LoadA (InpVar, 1)
	/* check header format */
    And(Literal, 0x1F)
	Subtract(Literal, 0x0C)
	RetNotZ  /*return if wrong format*/
    /* check the target address */
    CheckList(InpVar, 2, 2, Literal, 0x10 CSLIST 0xFE)
	RetNeg /*return if no match*/
	/* check if it is a request */
    LoadA(InpVar, 4)
	And(Literal, 0x40)
	RetNotZ /* return if not a request */
    /* check if the mode is supported */
    CheckList(InpVar, 4, 5, Literal,
        0x12 CSLIST 
        0x14 CSLIST 0x17 CSLIST
        0x19 CSLIST 0x20)
	JumpNeg(NegativeResp) /* not supported: negative response */
	ComputedCall(Mode12)  /* mode supported: process it */
	Call(PositiveResp)    /* and prepare positive response */
EndFunction

Function (PositiveResp)	
	Move(Literal, 4, OutpVar, 0) /*default response length*/
	Move(InpVar, 1, OutpVar, 1)  /* first three bytes are header */
	Move(InpVar, 3, OutpVar, 2)
	Move(Literal, 0x10, OutpVar, 3)
	LoadA(InpVar, 4)
	Or(Literal, 0x40) /*mode: response*/
	StoreA(OutpVar, 4)
EndFunction

Function(NegativeResp) /* bytes*/
	Call (PositiveResp)
	Move(Literal, 0x7F, OutpVar, 4) /*negative response*/
	LoadA( InpVar, 0 )		/*Length of input the message*/
	Add(Literal, 2)
	Minimum(Literal, 11)
	StoreA(OutpVar, 0)	/*length of neg. response */
	MoveAX
	MoveI(Literal, 0x11, OutpVar, 0) /* response code to the last byte */
	Subtract(Literal, 5)	/* - Header length */
	RetNegZ               /* return if nothing to copy*/
	RepeatA	/* Is the number of bytes to copy */
		Move(InpVar,4, OutpVar,5)
EndFunction
/* dummy mode processing functions */
Function( Mode12 )
	Move(Literal, 0x12, VPIO, 0)
EndFunction
Function( Mode14 )
	Move(Literal, 0x14, VPIO, 0)
EndFunction
Function( Mode17 )
	Move(Literal, 0x17, VPIO, 0)
EndFunction
Function( Mode19 )
	Move(Literal, 0x19, VPIO, 0)
EndFunction
Function( Mode20 )
	Move(Literal, 0x20, VPIO, 0)
EndFunction
EndScript


#include <stdio.h> /*for printf*/
short dummy_VPO(unsigned ionum, outfunc_arg_t arg)
{
    (void)ionum; /*not used*/
    printf("In mode %02X\n", arg);
    return 0; /*always success*/
}

static inpvar_t const myinp[]={5, 0x6C, 0xFE, 0xF0, 0x14, 0x39, 0xAA, 0xAB, 0xAC};


const sample_t SAE_example_svirmdata = {
    named_item(SAE_example),
    {myinp, nelem(myinp)},
    8, /*temps*/
    13, /*outputs*/
    NULL,
    dummy_VPO
};
